'use strict';

var AppName = 'GameTZ Go';
var AppVersionName = '0.2.9';
var AppVersion = 'GThf4a6NwAG'; // JSON server code
var BaseURL = 'https://gametz.com';

var TestingOptions = false;
var TestBaseURL = 'http://gametz.test';

var ExposeWhenLandScape = 'large';
var ExposeWhenPortrait = '(min-width:42424px)';
var ExposeWhenDisabled = '(min-width:76800px)';

var ImageHost = 'https://images.kenyonhill.com';

// configure moment relative time
moment.locale('en', {
  relativeTime: {
    future: "in %s",
    past: "%s ago",
    s: "%d sec",
    m: "a minute",
    mm: "%d minutes",
    h: "an hour",
    hh: "%d hours",
    d: "a day",
    dd: "%d days",
    M: "a month",
    MM: "%d months",
    y: "a year",
    yy: "%d years"
  }
});

angular.module('gametz', ['ionic', 'ngStorage', 'angularMoment', 'ngMessages', 'gametz.controllers'])

    .run(function($ionicPlatform, $rootScope) {
	$ionicPlatform.ready(function() {
	    // Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
	    // for form inputs)
	    if (window.cordova && window.cordova.plugins.Keyboard) {
		cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
		cordova.plugins.Keyboard.disableScroll(true);
	    }
	    if (window.StatusBar) {
		// org.apache.cordova.statusbar required
		StatusBar.styleDefault();
	    }
	    $rootScope.$apply();
	});
    })

    .config(function($stateProvider, $urlRouterProvider) {
	$stateProvider
	
	    .state('app', {
		url: '/app',
		abstract: true,
		templateUrl: 'templates/menu.html',
		controller: 'AppCtrl'
	    })
	
	    .state('app.mentions', {
		cache: false,
		url: '/mentions',
		views: {
		    'menuContent': {
			templateUrl: 'templates/mentions.html'
		    }
		}
	    })

	    .state('app.search', {
		url: '/search',
		views: {
		    'menuContent': {
			templateUrl: 'templates/search.html'
		    }
		}
	    })

	    .state('app.chat', {
		cache: false,
		url: '/chat',
		views: {
		    'menuContent': {
			templateUrl: 'templates/chat.html'
		    }
		}
	    })

	    .state('app.replies', {
		cache: false,
		url: '/replies',
		views: {
		    'menuContent': {
			templateUrl: 'templates/replies.html'
		    }
		}
	    })
	
	    .state('app.offers', {
 		cache: false,
		url: '/offers',
		views: {
		    'menuContent': {
			templateUrl: 'templates/offers.html'
		    }
		}
	    })

	    .state('app.offer', {
		cache: false,
		url: '/offer/:id',
		views: {
		    'menuContent': {
			templateUrl: 'templates/offer.html'
		    }
		}
	    })

	    .state('app.pms', {
 		cache: false,
		url: '/pms',
		views: {
		    'menuContent': {
			templateUrl: 'templates/pms.html'
		    }
		}
	    })

	    .state('app.pm', {
		cache: false,
		url: '/pm/:convid',
		views: {
		    'menuContent': {
			templateUrl: 'templates/pm.html'
		    }
		}
	    })

	    .state('app.log', {
		cache: false,
		url: '/log',
		views: {
		    'menuContent': {
			templateUrl: 'templates/log.html'
		    }
		}
	    })

	    .state('app.forums', {
		url: '/forums',
		views: {
		    'menuContent': {
			templateUrl: 'templates/forums.html'
		    }
		}
	    })

	    .state('app.marketplace', {
		url: '/marketplace',
		views: {
		    'menuContent': {
			templateUrl: 'templates/marketplace.html'
		    }
		}
	    })
	
	    .state('app.forum', {
		url: '/forum/:area',
		views: {
		    'menuContent': {
			templateUrl: 'templates/forum.html'
		    }
		}
	    })
	
	    .state('app.topic', {
		url: '/topic/:ftuid',
		views: {
		    'menuContent': {
			templateUrl: 'templates/topic.html'
		    }
		}
	    })
	
	;
	
	// if none of the above states are matched, use this as the fallback
	$urlRouterProvider.otherwise('/app/marketplace');
    })

    // derived from ngcordova-wrapper-nativestorage
    .factory('myNativeStorage', ['$q', function ($q) {
	return {
	    remove: function(reference) {
		var q = $q.defer();
		NativeStorage.remove(reference, function(result) {q.resolve(result);}, function(error) {q.reject(error);});
		return q.promise;
	    },
	    setItem: function(reference, s) {
		var q = $q.defer();
		NativeStorage.setItem(reference, s, function(result) {q.resolve(result);}, function(error) {q.reject(error);});
		return q.promise;
	    },
	    getItem: function(reference) {
		var q = $q.defer();
		NativeStorage.getItem(reference, function(result) {q.resolve(result);}, function(error) {q.reject(error);});
		return q.promise;
	    },
	    clear: function() {
		var q = $q.defer();
		NativeStorage.clear(function(result) {
		    q.resolve(result);
		}, function(error) {
		    q.reject(error);
		});
		return q.promise;
	    }
	};
    }])

    .factory('websock', function($rootScope) {
	var ws;
	var URI;
	return {
	    disconnect: function() {
		if (ws) ws.close();
		if ($rootScope.debugMode) console.log('websock closed');		    
	    },
	    connect: function() {
		if (ws) ws.close();
		if (!$rootScope.login.signedOn) {
		    if ($rootScope.debugMode) console.log("websock did not connect because not signed on");
		    return null;
		}
		
		URI = ($rootScope.baseURL == 'http://gametz.test')?'ws://192.168.0.10:8080/':'wss://gametz.com:8082';

		ws = new ReconnectingWebSocket(URI);
		ws.onopen = function(){
		    if ($rootScope.debugMode) console.log("websock has been opened! "+ws.readyState);
		}
		ws.onclose = function(){
		    if ($rootScope.debugMode) console.log("websock has been closed! "+ws.readyState);
		}
		ws.onmessage = function(event){
		    if (event.data === "ready") {
			try {
			    if (!$rootScope.$storage.uuid || !$rootScope.$storage.wsNonce) {
				if ($rootScope.debugMode) console.log("websock ready without uuid and wsNonce!");
			    } else {
				ws.send(['auth', $rootScope.$storage.uuid, $rootScope.$storage.wsNonce, 0, 0].join(' '));
				if ($rootScope.debugMode) console.log("websock auth sent");
			    }
			} catch(exception) {
			    if ($rootScope.debugMode) console.log('ws.onmessage exception = '+exception);
			}
		    } else if (event.data.search(/^refresh /) != -1) {
			var cnt = event.data.split(" ");
			$rootScope.$emit('websock:setCounts', [cnt[1], cnt[2], cnt[3]]);
			if ($rootScope.debugMode) console.log("emit setCounts = "+cnt[1]+' '+cnt[2]+' '+cnt[3]);
		    } else if (event.data.search(/^notify2 /) != -1) {
			var data = event.data.replace(/^notify2 /, "").split(String.fromCharCode(1)); // C_KS_AFS
			$rootScope.$emit('websock:notify2', data[0], data[1], data[2]);
			if ($rootScope.debugMode) console.log("emit notify2 "+data[0]+' '+data[1]+' '+data[2]);
		    } else if (event.data.search(/^log /) != -1) {
			event.data = event.data.replace(/^\s+|\s+$/g, '');
			if ($rootScope.debugMode) console.log(event.data.replace(/^log /, ""));
		    } else {
			if ($rootScope.debugMode) console.log('websock got unknown: '+event.data);
		    }
		}
	    }
	}
    })

    .factory('chatsock', function($rootScope) {
	var ws;
	var URI;
	return {
	    disconnect: function() {
		if (ws) ws.close();
		if ($rootScope.debugMode) console.log('chatsock closed');		    
	    },
	    send: function(msg) {
		if (ws) {
		    if (ws) try { ws.send("send "+msg); } catch(exception) { console.log('chatsock send = '+exception); }
		}
	    },
	    connect: function() {
		if (ws) ws.close();
		if (!$rootScope.login.signedOn) {
		    if ($rootScope.debugMode) console.log("chatsock did not connect because not signed on");
		    return null;
		}
		
		URI = ($rootScope.baseURL == 'http://gametz.test')?'ws://192.168.0.10:8083/':'wss://gametz.com:8083';

		if ($rootScope.debugMode) console.log("chatsock connect "+URI);
		ws = new ReconnectingWebSocket(URI);
		ws.timeoutInterval = 10000; // max time in milliseconds to wait for a connection to succeed before closing and retrying.
		ws.onopen = function(){
		    if ($rootScope.debugMode) console.log("chatsock has been opened! "+ws.readyState);
		}
		ws.onclose = function(){
		    if ($rootScope.debugMode) console.log("chatsock has been closed! "+ws.readyState);
		}
		ws.onmessage = function(event){
		    //if ($rootScope.debugMode) console.log("chatsock got Message: "+event.data);
		    if (event.data === "ready") {
			try {
			    if (!$rootScope.$storage.uuid || !$rootScope.$storage.chatNonce) {
				if ($rootScope.debugMode) console.log("chatsock ready without uuid and wsNonce!");
			    } else {
				ws.send(['auth', $rootScope.$storage.uuid, $rootScope.$storage.chatNonce].join(' '));
				if ($rootScope.debugMode) console.log("chatsock auth sent");
			    }
			} catch(exception) {
			    if ($rootScope.debugMode) console.log('ws.onmessage exception = '+exception);
			}
			
		    } else if (event.data === "stop") {
			ws.close(); 
		    } else if (event.data.search(/^msg /) != -1) {
			$rootScope.$emit('chatsock:addMessage', event.data);
		    } else if (event.data.search(/^who /) != -1) {
			$rootScope.$emit('chatsock:updateWhoList', event.data);
		    } else {
			if ($rootScope.debugMode) console.log('chatsock got unknown: '+event.data);
		    }
		}
	    }
	}
    })

    .filter('hrefToJS', function ($sce) {
	return function (text) {
	    if (typeof text == 'undefined') text = '';
            var newString = text.replace(/href="([\S]+)"/g, "href=\"#\" onClick=\"window.open(encodeURI('$1'), '_system');return false;\"");
            return $sce.trustAsHtml(newString);
	}
    })

    .filter('sql2PMDdate', ['$filter', function($filter) {
	return function(data) {
	    if (!data) return data;
	    return data.replace(' ', 'T');
	};
    }
			   ])

	    
;

