'use strict';

angular.module('gametz.controllers', [])

    .controller('AppCtrl', function($scope, $state, $ionicModal, $ionicPopup, $http, $localStorage, myNativeStorage, $ionicHistory, $ionicLoading, $timeout, websock, $rootScope) {

	$rootScope.Testing = TestingOptions; // static, used in login/account templates
	$rootScope.$storage = $localStorage;
	if (TestingOptions) {
	    if (typeof $rootScope.$storage.debugMode == 'undefined') $rootScope.$storage.debugMode = false;
	    if (typeof $rootScope.$storage.baseURL == 'undefined' ||
		([BaseURL, TestBaseURL].indexOf($rootScope.$storage.baseURL) == -1))
		$rootScope.$storage.baseURL = BaseURL;
	} else {
	    $rootScope.$storage.baseURL = BaseURL;
	    $rootScope.$storage.debugMode = false;
	}
	$rootScope.currentAsideWhen = ExposeWhenLandScape;
	if (typeof $rootScope.$storage.exposeAsideWhen == 'undefined') $rootScope.$storage.exposeAsideWhen = $rootScope.currentAsideWhen;
	
	$rootScope.consoleLog = [];

	// Logging
	$scope.logger = function(text) {
	    var cleaned = text.replace($rootScope.jsonURLBase, ' jsonURLBase ');
	    cleaned = cleaned.replace('&hash='+encodeURIComponent($rootScope.$storage.hash), ' hash ');
	    cleaned = cleaned.replace('&av='+AppVersion, ' av ');
	    if ($rootScope.$storage.debugMode) console.log(cleaned);
	    if ($rootScope.$storage.debugMode) $rootScope.consoleLog.unshift( { when: Date.now(), text: cleaned } ); 
	};
	
	$rootScope.loginURL = function() {
	    if ($rootScope.$storage.username && $rootScope.$storage.hash) {
		return '&u='+encodeURIComponent($rootScope.$storage.username)+'&hash='+encodeURIComponent($rootScope.$storage.hash);
	    } else {
		return '';
	    }
	};

	// Storage
	$scope.storageRemove = function(kv) {
	    $scope.logger('storageRemove '+kv);
	    delete $rootScope.$storage[kv];
	    if (typeof NativeStorage !== 'undefined') myNativeStorage.remove(kv);
	};

	$scope.syncStorage = function() {
	    if (typeof NativeStorage == 'undefined') return;
	    $rootScope.storedKeys.forEach(function(kv) {
		if (typeof $rootScope.$storage[kv] == 'undefined') {
		    // Try to restore from native
		    myNativeStorage.getItem(kv).then(function (value) {
			$scope.logger('syncStorage restored '+kv+' from native storage.');
			$rootScope.$storage[kv] = value;
			// Try to login now...
			if (!$rootScope.login.signedOn) $scope.autoLogin();
		    }); // ignore error case
		} else {
		    // Ensure important vars are backed-up in native
		    myNativeStorage.getItem(kv).then(function (value) {
			if (value !== $rootScope.$storage[kv]) {
			    $scope.logger('syncStorage updated '+kv+' in native storage.');
			    myNativeStorage.setItem(kv, $rootScope.$storage[kv]);
			}
		    }, function (error) {
			// Assume not found (error.code == 2), so set it.
			$scope.logger('syncStorage saved new '+kv+' to native storage.');
			myNativeStorage.setItem(kv, $rootScope.$storage[kv]);
                    });
		}
	    });
	};
	
	// Initialize some variables
	$rootScope.AppVersionName = AppVersionName;
	$rootScope.AppName = AppName;

	$rootScope.appState = 'foreground';
	
	$rootScope.login = { };

	$rootScope.storedKeys = ['username', 'hash', 'uuid', 'wsNonce', 'chatNonce', 'exposeAsideWhen'];
	$rootScope.jsonURLBase = '?A=JSON&av='+AppVersion;
	$rootScope.login.signedOn = false;
	$rootScope.online = true;
	$scope.buttons = { plus: false };


	$scope.sendScore = function(fmuid, uuid, key, val) {
	    $http({
		url: $rootScope.$storage.baseURL,
		method: 'POST',
		data: {
		    'u'      : $rootScope.$storage.username,
		    'hash'   : $rootScope.$storage.hash,
		    'A'      : 'Scorer',
		    'itable' : 'ForumMessages',
		    'iuid'   : fmuid,
		    'uuid'   : uuid,
		    'key'    : key,
		    'val'    : val,
		}
	    }).then(function(data) {
		if (data.data.error) {
		    $rootScope.error = data.data.error;
		    if (data.data.badsignon) { $scope.badSignOn(); }
		} else {
		    $rootScope.$broadcast('post.newScore', fmuid, data.data);
		}
	    }, function(data) { 
		$rootScope.error = "Failed to Score Forum Message!";
	    });
	};


	// Fixes annoying CORS error
	$http.defaults.headers.post = { 'Content-Type': 'application/x-www-form-urlencoded' };
	// parameterizes angulars JSON'd vars so my backend will see it normally (CGI.pm can deal)
	$http.defaults.transformRequest = function(data){
	    if (data === undefined) return data;
	    var buffer = [];
	    for ( var name in data ) {
                if (!data.hasOwnProperty(name)) continue;
                var value = data[name];
                buffer.push(
                    encodeURIComponent(name) +
                        "=" +
                        encodeURIComponent((value == null)?"" : value )
                );
            }
	    var source = buffer
                .join( "&" )
                .replace( /%20/g, "+" );
	    $scope.logger("xform = "+source);
            return (source);
	};

	// Menu counts
	$scope.newPMCount = 0;
	$scope.newOfferCount = 0;
	$scope.mentionCount = 0;

	$scope.updateCounts = function(data) {
	    if (typeof data !== 'undefined') {
		if (typeof data.NewPms       !== 'undefined') $scope.newPMCount    = data.NewPms;
		if (typeof data.NewOfferMsgs !== 'undefined') $scope.newOfferCount = data.NewOfferMsgs;
		if (typeof data.Mentions     !== 'undefined') $scope.mentionCount  = data.Mentions;
	    }
	};
	
	// websocket events
	$rootScope.$on("websock:setCounts", function(event, counts){
	    $scope.newPMCount = counts[0];
	    $scope.newOfferCount = counts[1];
	    $scope.mentionCount = counts[2];
	    $scope.$apply();
	});

	$rootScope.$on("websock:notify2", function(event, type, id, text){
	    var goplace = null;
	    var goparams = {};
	    
	    $scope.logger('notify2('+type+', '+id+', '+text+')');
	    if ($rootScope.appState != 'foreground') {
		$scope.logger('notify2 not in foreground');
		return null;
	    }
	    
	    if (type == 'pm') {
		goplace = 'app.pm';
		goparams['convid'] = id;
	    } else if (type == 'mention') {
		goplace = 'app.mentions';
	    } else if (type == 'offer') {
		goplace = 'app.offer';
		goparams['id'] = id;
	    } 
	    var popbuttons = [ { text: 'Dismiss', type: 'button-dark' } ];
	    if (goplace) {
		popbuttons.push( { text: 'View', type: 'button-positive',
				   onTap: function(e) {
				       $ionicHistory.clearCache().then(function(){
					   $state.go(goplace, goparams) });
				   }
				 });
	    }
	    
	    var notifyPopup = $ionicPopup.show({
		title: type,
		template: text,
		scope: $scope,
		buttons: popbuttons
	    });

	    $timeout(function() {
		notifyPopup.close(); //close the popup after 10 seconds
	    }, 10000);
	    
	});

	// autologin
	$scope.autoLogin = function() {
	    if ($rootScope.online && !$rootScope.login.signedOn && $rootScope.$storage.username &&
		$rootScope.$storage.hash && $rootScope.$storage.uuid && $rootScope.$storage.wsNonce &&
		$rootScope.$storage.chatNonce) {
		$rootScope.login.signedOn = true;
		websock.connect();
		$scope.syncStorage();
	    }
	};
	$scope.autoLogin();
	
	// Push Notificaions
	var push;
	$scope.unregisterNotify = function() {
	    $scope.logger('unregister');
	    if (typeof PushNotification == 'undefined') return null;
	    
	    push.unregister(function() {
		$scope.logger('unregister success');
		var url = $rootScope.$storage.baseURL+$rootScope.jsonURLBase+'&type=NotificationUnregistration&device='+encodeURIComponent($rootScope.login.registrationId);
		$scope.logger('registratrion url = '+url);
		$http.get(url).
		    success(function(data, status, headers, config) {
			$scope.logger("notificationSetup registration - success");
		    }).
		    error(function(err) {
			if ($rootScope.$storage.debugMode) console.error('notificationSetup registration - error', err);
		    });
	    }, function() {
		$scope.logger('unregister error');
	    });
	};

	$scope.setupWhenDeviceReady = function() {
	    $scope.logger('setupWhenDeviceReady');
	    
	    if (device.platform === 'Android') { // back button
		document.addEventListener("backbutton", function(e){
		    $ionicHistory.goBack();
		}, false);
	    }

	    // appState
	    document.addEventListener("pause", onPause, false);
	    function onPause() {
		$rootScope.appState = 'background';
		$scope.logger('appState = '+$rootScope.appState);
	    }
	    document.addEventListener("resume", onResume, false);
	    function onResume() {
		$rootScope.appState = 'foreground';
		$scope.logger('appState = '+$rootScope.appState);
	    }
	    $scope.setupNotify();
	    $scope.syncStorage();

	    screen.orientation.addEventListener('change', $scope.setupScreenLock, false);
	};

	$scope.setupScreenLock = function() {
	    if (screen.orientation.type.indexOf('portrait') !== -1) {
		$rootScope.currentAsideWhen = ExposeWhenPortrait;
	    } else {
		$rootScope.currentAsideWhen = ExposeWhenLandScape;
	    }
	    if ($rootScope.$storage.exposeAsideWhen !== ExposeWhenDisabled) {
		$scope.$apply(function () {
		    $rootScope.$storage.exposeAsideWhen = $rootScope.currentAsideWhen;
		});
	    }
	    $scope.logger('setupScreenLock: type = '+screen.orientation.type+', exposeAsideWhen = '+$rootScope.$storage.exposeAsideWhen);
	};
	
	$scope.setupNotify = function() {
	    $scope.logger('setupNotify');

	    document.addEventListener("offline", onOffline, false);
	    function onOffline() {
		$rootScope.online = false;
		if (!$rootScope.error.match(/offline/)) $rootScope.error = $rootScope.error+' Network is offline.';
		$scope.logger('offline');
	    }
	    document.addEventListener("online", onOnline, false);
	    function onOnline() {
		$rootScope.online = true;
		websock.connect();
		$scope.logger('online');
	    }
	
	    if (typeof PushNotification == 'undefined') {
		$scope.logger('setupNotify PushNotification undefined!');
		return null;
	    }
	    if (!$rootScope.login.signedOn) {
		$scope.logger('setupNotify not signed on!');
		return null;
	    }

	    $scope.logger('setupNotify init');
	    push = PushNotification.init({
		"android": {
		    "iconColor": "purple"
		},
		"ios": {
		    "alert":      "true",
		    "sound":      "true",
		    "badge":      "true",
		    "clearBadge": "true"
		},
		"windows": {}
	    });
    
	    push.on('registration', function(data) {
		$rootScope.login.registrationId = data.registrationId;
		var url = $rootScope.$storage.baseURL+$rootScope.jsonURLBase+'&type=NotificationRegistration'+$rootScope.loginURL()+'&device='+encodeURIComponent(data.registrationId)+'&plat='+device.platform;
		$scope.logger('registratrion url = '+url);
		$http.get(url).
		    success(function(data, status, headers, config) {
			$scope.logger("notificationSetup registration - success");
		    }).
		    error(function(err) {
			if ($rootScope.$storage.debugMode) console.error('notificationSetup registration - error', err);
		    });
	    });
	    
	    push.on('notification', function(data) {
		$scope.logger("notificationSetup - notification event");
		$scope.logger(JSON.stringify(data));

		// this assumes pm only, NYI offer/mention cases
		// also, could go to pm by id once that is passed back in data
		if (data.additionalData.foreground) {
		    // do nothing because websock handles this?
		    $scope.logger("do nothing because foreground");
		} else {
		    var goplace = null;
		    var goparams = {};
		    switch(data.additionalData.type) {
		    case 'pm':
			goplace = 'app.pm';
			goparams['convid'] = data.additionalData.id;
			break;
		    case 'offer':
			goplace = 'app.offer';
			goparams['id'] = data.additionalData.id;
			break;
		    case 'mention':
			goplace = 'app.mentions';
			break;
		    default: // 'other'
			
		    }
		    if (goplace) {
			$scope.logger('goplace ='+goplace);
			$ionicHistory.clearCache().then(function(){ $state.go(goplace, goparams); });
		    }
		}
		
		push.finish(function() { // helps ios know it worked or something
		    $scope.logger("processing of push data is finished");
		});
		
	    });
	    
	    push.on('error', function(e) {
		$scope.logger("notificationSetup - push error");
	    });
	};

	document.addEventListener("deviceready", $scope.setupWhenDeviceReady, false);

	// Create the login modal that we will use later
	$ionicModal.fromTemplateUrl('templates/login.html', {
	    scope: $scope
	}).then(function(modal) {
	    $scope.loginModal = modal;
	});

	// Create the create account modal that we will use later
	$ionicModal.fromTemplateUrl('templates/create.html', {
	    scope: $scope
	}).then(function(modal) {
	    $scope.createModal = modal;
	});

	// Triggered in the login modal to close it
	$scope.closeLogin = function() {
	    $rootScope.login.error = null;
	    $scope.loginModal.hide();
	};
	// Triggered in the login modal to close it
	$scope.closeCreate = function() {
	    $scope.createModal.hide();
	};

	$scope.createAccount = function() {
	    $scope.loginModal.hide();
	    $scope.createModal.show();
	};
	
	// Open the login modal
	$scope.loginScreen = function() {
	    $scope.loginModal.show();
	};

	// Create the account modal that we will use later
	$ionicModal.fromTemplateUrl('templates/account.html', {
	    scope: $scope
	}).then(function(modal) {
	    $scope.accountModal = modal;
	});
	
	// Triggered in the login modal to close it
	$scope.closeAccount = function() {
	    $scope.accountModal.hide();
	};

	// Open the account modal
	$scope.accountScreen = function() {
	    $scope.accountModal.show();
	};

	// Open the user modal
	$scope.userPage = function(uname) {
	    if (typeof(uname) === "undefined" || uname === 'system') { return null; }
	    $ionicModal.fromTemplateUrl('templates/user.html', {
		scope: $scope
	    }).then(function(modal) {
		$scope.userModal = modal;
		$scope.userModal.show();
	    });
	    $rootScope.userPageUName = uname;
	};

	$scope.closeUser = function() {
	    $scope.userModal.remove();
	};

	// Edit Post
	$scope.editPost = function(fmuid, RawMessage, FirstPost, HasDisable, NoScores, NoReplies, RawSubject) {
//	    $scope.logger('ns = '+NoScores+', nr = '+NoReplies);
	    $scope.fmuid = fmuid;
	    $scope.FirstPost = FirstPost;
	    $scope.HasDisable = HasDisable;
	    $scope.m = { noscores:  (NoScores?true:null),
			 noreplies: (NoReplies?true:null),
			 subject:   RawSubject,
			 text:      RawMessage,
		       };
//	    $scope.logger('m.ns = '+$scope.m.noscores+', nr = '+$scope.m.noreplies);
	    $ionicModal.fromTemplateUrl('templates/editpost.html', {
		scope: $scope
	    }).then(function(modal) {
		$scope.editPostModal = modal;
		$scope.editPostModal.show();
	    });
	};
	
	$scope.closeEditPost = function() {
	    $scope.editPostModal.remove();
	};

	// New Topic
	$scope.newtopic = function(area, DisplayArea, HasDisable) {
	    $scope.area = area;
	    $scope.DisplayArea = DisplayArea;
	    $scope.HasDisable = HasDisable;
	    $ionicModal.fromTemplateUrl('templates/newtopic.html', {
		scope: $scope
	    }).then(function(modal) {
		$scope.newtopicModal = modal;
		$scope.newtopicModal.show();
	    });
	};
	
	$scope.closenewtopic = function() {
	    $scope.newtopicModal.remove();
	};

	// PM User
	$scope.userPM = function(uname, subject, qtext) {
	    $scope.subject = { text: subject };
	    $scope.pmMsg = { text: null };
	    if (qtext.length > 0) $scope.pmMsg.text = "\n\n"+qtext;
	    if (typeof(uname) === "undefined" || uname === 'system') { return null; }
	    $ionicModal.fromTemplateUrl('templates/userpm.html', {
		scope: $scope
	    }).then(function(modal) {
		$scope.userPMModal = modal;
		$scope.userPMModal.show();
		if ($scope.userModal) $scope.userModal.remove();  // This replaces the user modal
	    });
	    $rootScope.toUName = uname;
	};

	$scope.closePMUser = function() {
	    $scope.userPMModal.remove();
	};
	
	$scope.badSignOn = function() {
	    $rootScope.login.signedOn = false;
	    $scope.storageRemove('hash');
	}
	
	$scope.doLogout = function() {
	    $rootScope.storedKeys.forEach(function(kv) { $scope.storageRemove(kv); });
	    $rootScope.login.signedOn = false;
	    $rootScope.login.password = null;
	    websock.disconnect();
	    $scope.closeAccount();
	    $scope.unregisterNotify();
	    $state.go('app.marketplace'); // NYI - better default page?
	    // Clear all cache and history
	    $timeout(function () {
		$ionicHistory.clearCache();
		$ionicHistory.clearHistory();
	    }, 1500)
	};

	
	// Perform the login action when the user submits the login form
	$scope.doLogin = function() {
	    $rootScope.login.error = null;
	    var url = $rootScope.$storage.baseURL+$rootScope.jsonURLBase+'&type=SignOn'+'&u='+encodeURIComponent($rootScope.$storage.username)+'&p='+encodeURIComponent($rootScope.login.password);
	    $scope.logger('doLogin url = '+url);
	    $http.get(url).
		success(function(data, status, headers, config) {
		    if (data.error) { 
			$rootScope.login.error = data.error;
			$rootScope.login.signedOn = false;
		    } else {
			$timeout(function () {
			    $ionicHistory.clearCache();
			    $ionicHistory.clearHistory();
			}, 1500)
			$rootScope.login.signedOn = true;
			$rootScope.$storage.hash = data.hash;
			$rootScope.$storage.uuid = data.uuid;
			$rootScope.$storage.wsNonce = data.wsNonce;
			$rootScope.$storage.chatNonce = data.chatNonce;
			$scope.syncStorage();
			$scope.updateCounts(data);
			websock.connect();
			$scope.closeLogin();
			$scope.setupNotify();
		    }
		}).
		error(function(data, status, headers, config) {
		    $rootScope.login.error = "Unable to connect for login";
		    if (!$rootScope.online) $rootScope.login.error = $rootScope.login.error+' Network is offline.';
		    $rootScope.login.signedOn = false;
		});
	};
    })

    .controller('log', function($scope, $rootScope, $localStorage) {
	$scope.localStorageReset = function() {
	    $rootScope.storedKeys.forEach(function(kv) {
		$scope.logger('delete $rootScope.$storage['+kv+']');
		delete $rootScope.$storage[kv];
	    });
	};
    })

    .controller('setSettings', function($scope, $window, $timeout, $rootScope, websock) {
	$scope.testServer = { checked: ($rootScope.$storage.baseURL === TestBaseURL)?true:false };
	$scope.debugging = { checked: $rootScope.$storage.debugMode };
	$scope.lockslider = { checked: ([ExposeWhenLandScape, ExposeWhenPortrait].indexOf($rootScope.$storage.exposeAsideWhen) == -1)?false:true };

	$scope.update = function() {
	    $rootScope.login.signedOn = false;
	    websock.disconnect();

	    var exposed = false;
	    if ($scope.lockslider.checked) {
		if ([ExposeWhenLandScape, ExposeWhenPortrait].indexOf($rootScope.$storage.exposeAsideWhen) == -1) {
		    $rootScope.$storage.exposeAsideWhen = $rootScope.currentAsideWhen;
		    exposed = true;
		}
	    } else {
		if ($rootScope.$storage.exposeAsideWhen !== ExposeWhenDisabled) {
		    $rootScope.$storage.exposeAsideWhen = ExposeWhenDisabled;
		    exposed = true;
		}
	    }
	    
	    if ($scope.testServer.checked) {
		if ($rootScope.$storage.baseURL !== TestBaseURL) {
		    $rootScope.$storage.baseURL = TestBaseURL;
		    exposed = true;
		}
	    } else {
		if ($rootScope.$storage.baseURL !== BaseURL) {
		    $rootScope.$storage.baseURL = BaseURL;
		    exposed = true;
		}
	    }
	    $scope.autoLogin();
	    
	    if ($scope.debugging.checked) {
		$rootScope.$storage.debugMode = true;
	    } else {
		$rootScope.$storage.debugMode = false;
	    }

	    if (exposed) $timeout(function() { $window.location.reload(); }, 100);
	}
    })

    .controller('forums', function($scope, $http, $state, $ionicLoading, $ionicListDelegate, $rootScope) {
	$scope.forums = [ ];
	$rootScope.error = null;
	$scope.nonetoshow = false;

	$rootScope.$on("topic.keepNew", function(event, area, ftuid){
	    for (var i=0; i < $scope.forums.length; i++) {
		if ($scope.forums[i].area === area) $scope.forums[i].isNew = true;
	    }
	});
	
	$scope.ignoreForum = function(area) {
	    $ionicLoading.show( { duration: 3000 });
	    $rootScope.error = null;
	    $http({
		url: $rootScope.$storage.baseURL,
		method: 'POST',
		data: {
		    'u'     : $rootScope.$storage.username,
		    'hash'  : $rootScope.$storage.hash,
		    'A'     : 'JSON',
		    'av'    : AppVersion,
		    'type'  : 'ForumIgnore',
		    'area'  : area,
		}
	    }).then(function(data) {
		if (data.data.error) {
		    $rootScope.error = data.data.error;
		    if (data.data.badsignon) { $scope.badSignOn(); }
		} else {
		    $ionicListDelegate.closeOptionButtons();
		    $scope.forums = [ ];
		    $scope.loadMore(0,0);
		}
		$ionicLoading.hide();
	    }, function(data) { 
		$rootScope.error = "Failed to Ignore Forum!";
		$ionicLoading.hide();
	    });
	};

	$scope.catchupForum = function(area) {
	    $ionicLoading.show({ duration: 3000 });
	    $rootScope.error = null;
	    $http({
		url: $rootScope.$storage.baseURL,
		method: 'POST',
		data: {
		    'u'     : $rootScope.$storage.username,
		    'hash'  : $rootScope.$storage.hash,
		    'A'     : 'JSON',
		    'av'    : AppVersion,
		    'type'  : 'ForumCatchup',
		    'area'  : area,
		}
	    }).then(function(data) {
		if (data.data.error) {
		    $rootScope.error = data.data.error;
		    if (data.data.badsignon) { $scope.badSignOn(); }
		} else {
		    for (var i=0; i < $scope.forums.length; i++) {
			if ($scope.forums[i].area === area) $scope.forums[i].isNew = false;
		    }
		    $ionicListDelegate.closeOptionButtons();
		    $rootScope.$broadcast('forum.catchup', area);
		}
		$ionicLoading.hide();
	    }, function(data) { 
		$rootScope.error = "Failed to catchup Forum!";
		$ionicLoading.hide();
	    });
	};

	$scope.go = function(area) {
	    $state.go('app.forum', { 'area': area });
	};
	
	$scope.loadMore = function(infin, refresh) {
	    if (refresh) {
		$scope.forums = [ ];
	    } else if (!infin) $ionicLoading.show( { duration: 3000 });
	    $scope.nonetoshow = false;
	    $rootScope.error = null;
	    var url = $rootScope.$storage.baseURL+$rootScope.jsonURLBase+'&type=forums'+$rootScope.loginURL();
	    $scope.logger('forums url = '+url);
	    $http.get(url).
		success(function(data, status, headers, config) {
		    if (data.error) {
			$rootScope.error = data.error;
			if (data.badsignon) { $scope.badSignOn(); }
		    } else {
			if (typeof data.forums != 'undefined') {
			    $scope.forums = data.forums;
			}
			if ($scope.forums.length == 0) { $scope.nonetoshow = true; }
		    }
		    if (refresh) $scope.$broadcast('scroll.refreshComplete');
		    else if (infin) $scope.$broadcast('scroll.infiniteScrollComplete'); 
		    else $ionicLoading.hide();
		}).
		error(function(data, status, headers, config) {
		    $rootScope.error = "Failed to get forums!";
		    if (!$rootScope.online) $rootScope.error = $rootScope.error+' Network is offline.';
		    if (refresh) $scope.$broadcast('scroll.refreshComplete');
		    else if (infin) $scope.$broadcast('scroll.infiniteScrollComplete'); 
		    else $ionicLoading.hide();
		});
	};
	$scope.loadMore(0,0);
    })

    .controller('forum', function($scope, $http, $stateParams, $ionicLoading, $rootScope) {
	$scope.topics = [ ];
	$scope.next = 0;
	$rootScope.error = null;
	$scope.area = $stateParams.area;
	$scope.moreItems = true;
	$scope.nonetoshow = false;
	$scope.DisplayArea = null;
	$scope.HasDisable = null;
	
	$scope.$on('$ionicView.enter', function(){
	    $scope.buttons.plus = true;
	});
	$scope.$on('$ionicView.leave', function(){
	    $scope.buttons.plus = false;
	});
	$rootScope.$on("post.new", function(event){
	    $scope.loadMore(0,1);
	});
	
	$rootScope.$on("topic.keepNew", function(event, area, ftuid){
	    if (area === $scope.area) {
		for (var i=0; i < $scope.topics.length; i++) {
		    if ($scope.topics[i].ID == ftuid) $scope.topics[i].isNew = true;
		}
	    }
	});
	$rootScope.$on("topic.clearNew", function(event, ftuid){
	    for (var i=0; i < $scope.topics.length; i++) {
		if ($scope.topics[i].ID == ftuid) $scope.topics[i].isNew = false;
	    }
	});
	$rootScope.$on("forum.catchup", function(event, area){
	    if (area === $scope.area) {
		for (var i=0; i < $scope.topics.length; i++) {
		    $scope.topics[i].isNew = false;
		}
	    }
	});

	$rootScope.plusButton = function() {
	    $scope.newtopic($scope.area, $scope.DisplayArea, $scope.HasDisable);
	};
	
	$scope.ignoreTopic = function(topic) {
	    $ionicLoading.show( { duration: 3000 });
	    $rootScope.error = null;
	    $http({
		url: $rootScope.$storage.baseURL,
		method: 'POST',
		data: {
		    'u'     : $rootScope.$storage.username,
		    'hash'  : $rootScope.$storage.hash,
		    'A'     : 'JSON',
		    'av'    : AppVersion,
		    'type'  : 'TopicIgnore',
		    'ftuid' : topic.ID,
		}
	    }).then(function(data) {
		if (data.data.error) {
		    $rootScope.error = data.data.error;
		    if (data.data.badsignon) { $scope.badSignOn(); }
		} else {
		    $scope.topics.splice($scope.topics.indexOf(topic), 1);
		}
		$ionicLoading.hide();
	    }, function(data) { 
		$rootScope.error = "Failed to ignore topic!";
		$ionicLoading.hide();
	    });
	};
	
	$scope.loadMore = function(infin, refresh) {
	    if (refresh) {
		$scope.topics = [ ];
		$scope.next = 0;
		$scope.moreItems = true;
	    } else if (!infin) $ionicLoading.show( { duration: 3000 });
	    $scope.nonetoshow = false;
	    $rootScope.error = null;
	    var url = $rootScope.$storage.baseURL+$rootScope.jsonURLBase+'&type=forum&area='+encodeURIComponent($scope.area)+$rootScope.loginURL()+'&next='+$scope.next;
	    $scope.logger('forum url = '+url);
	    $http.get(url).
		success(function(data, status, headers, config) {
		    if (data.error) {
			$rootScope.error = data.error;
			$scope.moreItems = false;
			if (data.badsignon) { $scope.badSignOn(); }
		    } else {
			$scope.DisplayArea = data.DisplayArea;
			$scope.HasDisable = data.HasDisable;
			if (typeof data.topics == 'undefined') {
			    $scope.moreItems = false;
			} else {
			    $scope.topics = $scope.topics.concat(data.topics);
			    if (data.topics.length < data.maxRecords) $scope.moreItems = false;
			    $scope.next += data.topics.length;
			}
			if ($scope.topics.length == 0) { $scope.nonetoshow = true; }
		    }
		    if (refresh) $scope.$broadcast('scroll.refreshComplete');
		    else if (infin) $scope.$broadcast('scroll.infiniteScrollComplete'); 
		    else $ionicLoading.hide();
		}).
		error(function(data, status, headers, config) {
		    $rootScope.error = "Failed to get forum topics!";
		    if (!$rootScope.online) $rootScope.error = $rootScope.error+' Network is offline.';
		    if (refresh) $scope.$broadcast('scroll.refreshComplete');
		    else if (infin) $scope.$broadcast('scroll.infiniteScrollComplete');
		    else $ionicLoading.hide();
		});
	};
	$scope.loadMore(0,0);
    })

    .controller('replies', function($scope, $http, $stateParams, $ionicLoading, $rootScope) {
	$scope.topics = [ ];
	$scope.next = 0;
	$rootScope.error = null;
	$scope.moreItems = true;
	$scope.nonetoshow = false;
	$scope.LastCheck = null;

	$rootScope.$on("topic.clearNew", function(event, ftuid){
	    for (var i=0; i < $scope.topics.length; i++) {
		if ($scope.topics[i].ID == ftuid) $scope.topics.splice($scope.topics.indexOf($scope.topics[i]), 1);
	    }
	    if ($scope.topics.length == 0) { $scope.nonetoshow = true; }	    
	});
	
	$scope.ignoreTopic = function(topic) {
	    $ionicLoading.show( { duration: 3000 });
	    $rootScope.error = null;
	    $http({
		url: $rootScope.$storage.baseURL,
		method: 'POST',
		data: {
		    'u'     : $rootScope.$storage.username,
		    'hash'  : $rootScope.$storage.hash,
		    'A'     : 'JSON',
		    'av'    : AppVersion,
		    'type'  : 'TopicIgnore',
		    'ftuid' : topic.ID,
		}
	    }).then(function(data) {
		if (data.data.error) {
		    $rootScope.error = data.data.error;
		    if (data.data.badsignon) { $scope.badSignOn(); }
		} else {
		    $scope.topics.splice($scope.topics.indexOf(topic), 1);
		}
		$ionicLoading.hide();
	    }, function(data) { 
		$rootScope.error = "Failed to ignore topic!";
		$ionicLoading.hide();
	    });
	};

	$scope.loadMore = function(infin, refresh) {
	    if (refresh) {
		$scope.topics = [ ];
		$scope.next = 0;
		$scope.moreItems = true;
	    } else if (!infin) $ionicLoading.show( { duration: 3000 });
	    $scope.nonetoshow = false;
	    $rootScope.error = null;
	    var url = $rootScope.$storage.baseURL+$rootScope.jsonURLBase+'&type=replies'+$rootScope.loginURL()+'&next='+$scope.next;
	    $scope.logger('replies url = '+url);
	    $http.get(url).
		success(function(data, status, headers, config) {
		    if (data.error) {
			$rootScope.error = data.error;
			$scope.moreItems = false;
			if (data.badsignon) { $scope.badSignOn(); }
		    } else {
			if (typeof data.topics == 'undefined') {
			    $scope.moreItems = false;
			} else {
			    $scope.topics = $scope.topics.concat(data.topics);
			    if (data.topics.length < data.maxRecords) $scope.moreItems = false;
			    $scope.next += data.topics.length;
			}
			if ($scope.topics.length == 0) { $scope.nonetoshow = true; }
			$scope.LastCheck = data.LastCheck;
		    }
		    if (refresh) $scope.$broadcast('scroll.refreshComplete');
		    else if (infin) $scope.$broadcast('scroll.infiniteScrollComplete'); 
		    else $ionicLoading.hide();
		}).
		error(function(data, status, headers, config) {
		    $rootScope.error = "Failed to get forum replies!";
		    if (!$rootScope.online) $rootScope.error = $rootScope.error+' Network is offline.';
		    if (refresh) $scope.$broadcast('scroll.refreshComplete');
		    else if (infin) $scope.$broadcast('scroll.infiniteScrollComplete'); 
		    else $ionicLoading.hide();
		});
	};
	$scope.loadMore(0,0);
    })

    .controller('marketplace', function($scope, $http, $ionicLoading, $rootScope) {
	$scope.topics = [ ];
	$scope.next = 0;
	$rootScope.error = null;
	$scope.moreItems = true;
	$scope.backstop = Math.round(+new Date()/1000);
	$scope.nonetoshow = false;
	
	$scope.ignoreTopic = function(topic) {
	    $ionicLoading.show( { duration: 3000 });
	    $rootScope.error = null;
	    $http({
		url: $rootScope.$storage.baseURL,
		method: 'POST',
		data: {
		    'u'     : $rootScope.$storage.username,
		    'hash'  : $rootScope.$storage.hash,
		    'A'     : 'JSON',
		    'av'    : AppVersion,
		    'type'  : 'TopicIgnore',
		    'ftuid' : topic.ID,
		}
	    }).then(function(data) {
		if (data.data.error) {
		    $rootScope.error = data.data.error;
		    if (data.data.badsignon) { $scope.badSignOn(); }
		} else {
		    $scope.topics.splice($scope.topics.indexOf(topic), 1);
		}
		$ionicLoading.hide();
	    }, function(data) { 
		$rootScope.error = "Failed to ignore topic!";
		$ionicLoading.hide();
	    });
	};
	
	$scope.loadMore = function(infin, refresh) {
	    if (refresh) {
		$scope.topics = [ ];
		$scope.next = 0;
		$scope.backstop = Math.round(+new Date()/1000);
		$scope.moreItems = true;
	    } else if (!infin) $ionicLoading.show( { duration: 3000 });
	    $scope.nonetoshow = false;
	    $rootScope.error = null;
	    var url = $rootScope.$storage.baseURL+$rootScope.jsonURLBase+'&type=marketplace'+$rootScope.loginURL()+'&backstop='+$scope.backstop+'&next='+$scope.next;
	    $scope.logger('marketplace url = '+url);
	    $http.get(url).
		success(function(data, status, headers, config) {
		    if (data.error) {
			$rootScope.error = data.error;
			$scope.moreItems = false;
			if (data.badsignon) { $scope.badSignOn(); }
			$scope.moreItems = false;
		    } else {
			if (typeof data.topics == 'undefined') {
			    $scope.moreItems = false;
			} else {
			    if (data.topics.length < data.maxRecords) $scope.moreItems = false;
			    $scope.next += data.topics.length;
			    $scope.topics = $scope.topics.concat(data.topics);
			}
			if ($scope.topics.length == 0) { $scope.nonetoshow = true; }
                    }
		    if (refresh) $scope.$broadcast('scroll.refreshComplete');
		    else if (infin) $scope.$broadcast('scroll.infiniteScrollComplete'); 
		    else $ionicLoading.hide();
		}).
		error(function(data, status, headers, config) {
		    $rootScope.error = "Failed to get Marketplace topics!";
		    if (!$rootScope.online) $rootScope.error = $rootScope.error+' Network is offline.';
		    if (refresh) $scope.$broadcast('scroll.refreshComplete');
		    else if (infin) $scope.$broadcast('scroll.infiniteScrollComplete');
		    else $ionicLoading.hide();
		});
	};
	$scope.loadMore(0,0);
    })

    .controller('pms', function($scope, $http, $ionicLoading, $rootScope) {
	$scope.conversations = [ ];
	$scope.next = 0;
	$rootScope.error = null;
	$scope.moreItems = true;
	$scope.nonetoshow = false;

	$rootScope.$on("pm.keepNew", function(event, pmid){
	    for (var i=0; i < $scope.conversations.length; i++) {
		if ($scope.conversations[i].ID == pmid) $scope.conversations[i].isNew = true;
	    }
	});

	$rootScope.$on("pm.clearNew", function(event, pmid){
	    for (var i=0; i < $scope.conversations.length; i++) {
		if ($scope.conversations[i].ID == pmid) $scope.conversations[i].isNew = false;
	    }
	});

	$scope.loadMore = function(infin, refresh) {
	    if (refresh) {
		$scope.conversations = [ ];
		$scope.next = 0;
		$scope.moreItems = true;
	    } else if (!infin) $ionicLoading.show( { duration: 3000 });
	    $scope.nonetoshow = false;
	    $rootScope.error = null;
	    var url = $rootScope.$storage.baseURL+$rootScope.jsonURLBase+'&type=pms'+$rootScope.loginURL()+'&next='+$scope.next;
	    $scope.logger('pms url = '+url);
	    $http.get(url).
		success(function(data, status, headers, config) {
		    if (data.error) {
			$rootScope.error = data.error;
			$scope.moreItems = false;
			if (data.badsignon) { $scope.badSignOn(); }
		    } else {
			if (typeof data.conversations == 'undefined') {
			    $scope.moreItems = false;
			} else {
			    if (data.conversations.length < data.maxRecords) $scope.moreItems = false;
			    $scope.next += data.conversations.length;
			    $scope.conversations = $scope.conversations.concat(data.conversations);
			}
			if ($scope.conversations.length == 0) { $scope.nonetoshow = true; }
                    }
		    if (refresh) $scope.$broadcast('scroll.refreshComplete');
		    else if (infin) $scope.$broadcast('scroll.infiniteScrollComplete');
		    else $ionicLoading.hide();
		}).
		error(function(data, status, headers, config) {
		    $rootScope.error = "Failed to get PM conversation list!";
		    if (!$rootScope.online) $rootScope.error = $rootScope.error+' Network is offline.';
		    if (refresh) $scope.$broadcast('scroll.refreshComplete');
		    else if (infin) $scope.$broadcast('scroll.infiniteScrollComplete');
		    else $ionicLoading.hide();
		});
	};
	$scope.loadMore(0,0);
    })

    .controller('offers', function($scope, $http, $ionicLoading, $rootScope) {
	$scope.offers = [ ];
	$scope.next = 0;
	$rootScope.error = null;
	$scope.moreItems = true;
	$scope.nonetoshow = false;

	$rootScope.$on("offer.keepNew", function(event, id){
	    for (var i=0; i < $scope.offers.length; i++) {
		if ($scope.offers[i].ID == id) $scope.offers[i].isNew = true;
	    }
	});

	$rootScope.$on("offer.clearNew", function(event, id){
	    for (var i=0; i < $scope.offers.length; i++) {
		if ($scope.offers[i].ID == id) $scope.offers[i].isNew = false;
	    }
	});

	$scope.loadMore = function(infin, refresh) {
	    if (refresh) {
		$scope.offers = [ ];
		$scope.next = 0;
		$scope.moreItems = true;
	    } else if (!infin) $ionicLoading.show( { duration: 3000 });
	    $scope.nonetoshow = false;
	    $rootScope.error = null;
	    var url = $rootScope.$storage.baseURL+$rootScope.jsonURLBase+'&type=offers'+$rootScope.loginURL()+'&next='+$scope.next;
	    $scope.logger('offers url = '+url);
	    $http.get(url).
		success(function(data, status, headers, config) {
		    if (data.error) {
			$rootScope.error = data.error;
			$scope.moreItems = false;
			if (data.badsignon) { $scope.badSignOn(); }
		    } else {
			if (typeof data.offers == 'undefined') {
			    $scope.moreItems = false;
			} else {
			    if (data.offers.length < data.maxRecords) $scope.moreItems = false;
			    $scope.next += data.offers.length;
			    $scope.offers = $scope.offers.concat(data.offers);
			}
			if ($scope.offers.length == 0) { $scope.nonetoshow = true; }
                    }
		    if (refresh) $scope.$broadcast('scroll.refreshComplete');
		    else if (infin) $scope.$broadcast('scroll.infiniteScrollComplete'); 
		    else $ionicLoading.hide();
		}).
		error(function(data, status, headers, config) {
		    $rootScope.error = "Failed to get Offer list!";
		    if (!$rootScope.online) $rootScope.error = $rootScope.error+' Network is offline.';
		    if (refresh) $scope.$broadcast('scroll.refreshComplete');
		    else if (infin) $scope.$broadcast('scroll.infiniteScrollComplete'); 
		    else $ionicLoading.hide();
		});
	};
	$scope.loadMore(0,0);
    })

    .controller('offer', function($scope, $http, $stateParams, $location, $ionicScrollDelegate, $state, $ionicLoading, $rootScope) {
	$scope.messages = [ ];
	$rootScope.error = null;
	$scope.offerid = $stateParams.id;
	$scope.msg = { text: null };
	$scope.fromUser = null;
	$scope.toUser = null;
	$scope.fromItems = null;
	$scope.toItems = null;
	$scope.status = null;
	$scope.expires = null;
	$scope.created = null;
	$scope.summary = null;
	$scope.link = null;
	$scope.moreItems = true;
	$scope.next = 0;
	
	$scope.sendMsg = function() {
	    $ionicLoading.show( { duration: 3000 });
	    $rootScope.error = null;
	    $http({
		url: $rootScope.$storage.baseURL,
		method: 'POST',
		data: {
		    'u'     : $rootScope.$storage.username,
		    'hash'  : $rootScope.$storage.hash,
		    'A'     : 'JSON',
		    'av'    : AppVersion,
		    'type'  : 'SendOfferMsg',
		    'offer' : $scope.offerid,
		    'msg'   : $scope.msg.text,
		}
	    }).then(function(data) {
		if (data.data.error) {
		    $rootScope.error = data.data.error;
		    if (data.data.badsignon) { $scope.badSignOn(); }
		} else {
		    $scope.msg = { text: null };
		    $scope.messages = [ ];
		    $scope.next = 0;
		    $scope.loadMore(0,0);
                }
		$ionicLoading.hide();
	    }, function(data) { 
		$rootScope.error = "Failed to send Offer Message!";
		$ionicLoading.hide();
	    });
	};

	$scope.keepNew = function() {
	    $ionicLoading.show( { duration: 3000 });
	    $rootScope.error = null;
	    $http({
		url: $rootScope.$storage.baseURL,
		method: 'POST',
		data: {
		    'u'     : $rootScope.$storage.username,
		    'hash'  : $rootScope.$storage.hash,
		    'A'     : 'JSON',
		    'av'    : AppVersion,
		    'type'  : 'OfferKeepNew',
		    'offer' : $scope.offerid,
		}
	    }).then(function(data) {
		if (data.data.error) {
		    $rootScope.error = data.data.error;
		    if (data.data.badsignon) { $scope.badSignOn(); }
		} else {
		    $scope.updateCounts(data.data);
		    $rootScope.$emit('offer.keepNew', $scope.offerid);
                    $state.go('app.offers');
		}
		$ionicLoading.hide();
	    }, function(data) { 
		$rootScope.error = "Failed to Keep New Offer!";
		$ionicLoading.hide();
	    });
	};
	
	$scope.loadMore = function(infin, refresh) {
	    if (refresh) {
		$scope.messages = [ ];
		$scope.next = 0;
		$scope.moreItems = true;
	    } else if (!infin) $ionicLoading.show( { duration: 3000 });
	    $rootScope.error = null;
	    var url = $rootScope.$storage.baseURL+$rootScope.jsonURLBase+'&type=offer&id='+$scope.offerid+$rootScope.loginURL()+'&next='+$scope.next;
	    $scope.logger('offer url = '+url);
	    $http.get(url).
		success(function(data, status, headers, config) {
		    if (data.error) {
			$rootScope.error = data.error;
			$scope.moreItems = false;
			if (data.badsignon) { $scope.badSignOn(); }
		    } else {
			$scope.fromUser    = data.FromUser;
			$scope.toUser      = data.ToUser;
			$scope.fromItems   = data.FromItems;
			$scope.toItems     = data.ToItems;
			$scope.status      = data.Status;
			$scope.expires     = data.Expires;
			$scope.created     = data.Created;
			$scope.tradeStatus = data.TradeStatus;
			$scope.summary     = data.Summary;
			$scope.weblink     = data.Link;
			$scope.updateCounts(data);
			
			$rootScope.$broadcast('offer.clearNew', $scope.offerid);
			if (typeof data.messages == 'undefined') {
			    $scope.moreItems = false;
			} else {
			    if (data.messages.length < data.maxRecords) $scope.moreItems = false;
			    $scope.next += data.messages.length;
			    $scope.messages = $scope.messages.concat(data.messages);
			}
                    }
		    if (refresh) $scope.$broadcast('scroll.refreshComplete');
		    else if (infin) $scope.$broadcast('scroll.infiniteScrollComplete'); 
		    else $ionicLoading.hide();
		}).
		error(function(data, status, headers, config) {
		    $rootScope.error = "Failed to get Offer info!";
		    if (!$rootScope.online) $rootScope.error = $rootScope.error+' Network is offline.';
		    if (refresh) $scope.$broadcast('scroll.refreshComplete');
		    else if (infin) $scope.$broadcast('scroll.infiniteScrollComplete'); 
		    else $ionicLoading.hide();
		});
	};
	$scope.loadMore(0,0);
    })

    .controller('mentions', function($scope, $http, $ionicLoading, $ionicActionSheet, $rootScope) {
	$scope.imghost = ImageHost;
	$scope.users = [ ];
	$scope.mentions = [ ];
	$scope.next = 0;
	$rootScope.error = null;
	$scope.moreItems = true;
	$scope.nonetoshow = false;

	$rootScope.$on("post.newScore", function(event, fmuid, score){
	    for (var i=0; i < $scope.mentions.length; i++) {
		if ($scope.mentions[i].FMUID == fmuid) $scope.mentions[i].Score = score;
	    }
	});

	$scope.postActionsheet = function(mention) {

	    var buttons = [ { text: 'Quoted PM to '+$scope.users[mention.Creator].UName, act: 'qpm' } ];
	    if (typeof mention.Score != 'undefined') {
		if (!mention.PlusOnly) buttons.push( { text: '<span class="red"><i class="icon ion-minus"></i> Neg</span>', act: 'neg'  } );
		buttons.push( { text: '<span class="green"><i class="icon ion-plus"></i> Plus</span>', act: 'plus'  } );
	    }
	    
    	    $ionicActionSheet.show({
		titleText: 'Mention Actions',
		buttons: buttons,
		cancelText: 'Cancel',
		buttonClicked: function(index) {
		    var act = buttons[index].act;
		    if (act === 'qpm') {
			$scope.userPM($scope.users[mention.Creator].UName, mention.RawSubject, mention.Replyify);
		    } else if (act === 'neg') {
			$scope.sendScore(mention.FMUID, $rootScope.$storage.uuid, mention.ScoreKey, -1);
		    } else if (act === 'plus') {
			$scope.sendScore(mention.FMUID, $rootScope.$storage.uuid, mention.ScoreKey, 1);
		    }
		    return true;
		}
	    });
	};

	$scope.loadMore = function(infin, refresh) {
	    if (refresh) {
		$scope.mentions = [ ];
		$scope.next = 0;
		$scope.moreItems = true;
	    } else if (!infin) $ionicLoading.show( { duration: 3000 });
	    $scope.nonetoshow = false;
	    $rootScope.error = null;
	    var url = $rootScope.$storage.baseURL+$rootScope.jsonURLBase+'&type=mentions'+$rootScope.loginURL()+'&next='+$scope.next;
	    $scope.logger('Mentions url = '+url);
	    $http.get(url).
		success(function(data, status, headers, config) {
		    if (data.error) {
			$rootScope.error = data.error;
			$scope.moreItems = false;
			if (data.badsignon) { $scope.badSignOn(); }
		    } else {
			for (var uuid in data.users) {
			    $scope.users[uuid] = data.users[uuid];
			}
			$scope.updateCounts(data);
			if (typeof data.mentions == 'undefined') {
			    $scope.moreItems = false;
			} else {
			    if (data.mentions.length < data.maxRecords) $scope.moreItems = false;
			    $scope.next += data.mentions.length;
			    $scope.mentions = $scope.mentions.concat(data.mentions);
			}
			if ($scope.mentions.length == 0) { $scope.nonetoshow = true; }
                    }
		    if (refresh) $scope.$broadcast('scroll.refreshComplete');
		    else if (infin) $scope.$broadcast('scroll.infiniteScrollComplete'); 
		    else $ionicLoading.hide();
		}).
		error(function(data, status, headers, config) {
		    $rootScope.error = "Failed to get Mentions!";
		    if (!$rootScope.online) $rootScope.error = $rootScope.error+' Network is offline.';
		    if (refresh) $scope.$broadcast('scroll.refreshComplete');
		    else if (infin) $scope.$broadcast('scroll.infiniteScrollComplete'); 
		    else $ionicLoading.hide();
		});
	};
	$scope.loadMore(0,0);
    })

    .controller('search', function($scope, $http, $ionicLoading, $ionicActionSheet, $rootScope) {
	$scope.imghost = ImageHost;
	$scope.users = [ ];
	$scope.results = [ ];
	$scope.next = 0;
	$rootScope.error = null;
	$scope.m = { query: null, selectedSection: 'Forum' };
	$scope.moreItems = true;
	$scope.nonetoshow = false;
	$scope.sections = [ 'Forum', 'Users' ];

	$rootScope.$on("post.newScore", function(event, fmuid, score){
	    for (var i=0; i < $scope.results.length; i++) {
		if ($scope.results[i].FMUID == fmuid) $scope.results[i].Score = score;
	    }
	});

	$scope.postActionsheet = function(result) {

	    var buttons = [ { text: 'Quoted PM to '+$scope.users[result.Creator].UName, act: 'qpm' } ];
	    if (typeof result.Score != 'undefined') {
		if (!result.PlusOnly) buttons.push( { text: '<span class="red"><i class="icon ion-minus"></i> Neg</span>', act: 'neg'  } );
		buttons.push( { text: '<span class="green"><i class="icon ion-plus"></i> Plus</span>', act: 'plus'  } );
	    }
	    
    	    $ionicActionSheet.show({
		titleText: 'Result Actions',
		buttons: buttons,
		cancelText: 'Cancel',
		buttonClicked: function(index) {
		    var act = buttons[index].act;
		    if (act === 'qpm') {
			$scope.userPM($scope.users[result.Creator].UName, result.RawSubject, result.Replyify);
		    } else if (act === 'neg') {
			$scope.sendScore(result.FMUID, $rootScope.$storage.uuid, result.ScoreKey, -1);
		    } else if (act === 'plus') {
			$scope.sendScore(result.FMUID, $rootScope.$storage.uuid, result.ScoreKey, 1);
		    }
		    return true;
		}
	    });
	};

	$scope.loadMore = function(infin, refresh) {
	    if (!$scope.m.query) {
		$scope.moreItems = false;
		$scope.nonetoshow = true;
		$scope.results = [ ];
		if (refresh) $scope.$broadcast('scroll.refreshComplete');
		return;
	    }
	    $scope.m.query.replace(/^\s\s*/, '').replace(/\s\s*$/, '');
	    if (refresh) {
		$scope.moreItems = true;
		$scope.results = [ ];
		$scope.next = 0;
	    } else if (!infin) {
		$ionicLoading.show( { duration: 3000 });
		$scope.moreItems = true;
		$scope.results = [ ];
		$scope.next = 0;
	    }
	    $scope.nonetoshow = false;
	    $rootScope.error = null;
	    $scope.logger('selected section = '+$scope.m.selectedSection);
	    var url = $rootScope.$storage.baseURL+$rootScope.jsonURLBase+'&type=search'+'&q='+encodeURIComponent($scope.m.query)+'&section='+encodeURIComponent($scope.m.selectedSection)+$rootScope.loginURL()+'&next='+$scope.next;
	    $scope.logger('Search url = '+url);
	    $http.get(url).
		success(function(data, status, headers, config) {
		    if (data.error) {
			$rootScope.error = data.error;
			$scope.moreItems = false;
			if (data.badsignon) { $scope.badSignOn(); }
		    } else {
			for (var uuid in data.users) {
			    $scope.users[uuid] = data.users[uuid];
			}
			if (typeof data.results == 'undefined') {
			    $scope.moreItems = false;
			} else {
			    if (data.results.length < data.maxRecords) $scope.moreItems = false;
			    $scope.next += data.results.length;
			    $scope.results = $scope.results.concat(data.results);
			}
			if ($scope.results.length == 0) { $scope.nonetoshow = true; }
                    }
		    if (refresh) $scope.$broadcast('scroll.refreshComplete');
		    else if (infin) $scope.$broadcast('scroll.infiniteScrollComplete'); 
		    else $ionicLoading.hide();
		}).
		error(function(data, status, headers, config) {
		    $rootScope.error = "Failed to get search results!";
		    if (!$rootScope.online) $rootScope.error = $rootScope.error+' Network is offline.';
		    if (refresh) $scope.$broadcast('scroll.refreshComplete');
		    else if (infin) $scope.$broadcast('scroll.infiniteScrollComplete'); 
		    else $ionicLoading.hide();
		});
	};
	$scope.loadMore(0,0);
    })

    .controller('topic', function($scope, $http, $stateParams, $location, $ionicScrollDelegate, $state, $ionicLoading, $ionicPopup, $ionicActionSheet, $rootScope) {
	$scope.users = [ ];
	$scope.messages = [ ];
	$scope.next = 0;
	$rootScope.error = null;
	$scope.moreItems = true;
	$scope.ftuid = $stateParams.ftuid;
	$scope.forumPost = { text: null };
	$scope.Subject = null;
	$scope.RawSubject = null;
	$scope.area = null;
	
	$rootScope.$on("post.update", function(event, data) {
 	    for (var i=0; i < $scope.messages.length; i++) {
 		if ($scope.messages[i].ID == data.ID) {
		    $scope.messages[i].Message = data.Message;
		    if (data.FirstPost) {
			if (data.Subject) $scope.Subject = data.Subject;
			if (data.HasDisable) {
//			    $scope.logger('pu: ns = '+data.NoScores+', nr = '+data.NoReplies);
			    $scope.messages[i].Score = data.Score;
			    $scope.messages[i].PlusOnly = data.PlusOnly;
			    $scope.messages[i].NoScores = data.NoScores?true:false;
			    $scope.messages[i].NoReplies = data.NoReplies?true:false;
//			    $scope.logger('pu: ns = '+$scope.messages[i].NoScores+', nr = '+$scope.messages[i].NoReplies);
			    $scope.NoPost = data.NoPost;
			}
		    }
		    $scope.messages[i].RawMessage = data.RawMessage;
		    $scope.messages[i].Replyify = data.Replyify;
		}
 	    }
	});
	
	$rootScope.$on("post.newScore", function(event, fmuid, score) {
	    for (var i=0; i < $scope.messages.length; i++) {
		if ($scope.messages[i].ID == fmuid) $scope.messages[i].Score = score;
	    }
	});
	
	$scope.postActionsheet = function(message) {

	    var buttons = [ { text: '@'+$scope.users[message.Creator].UName, act: 'mention' },
			    { text: 'Quoted forum reply', act: 'qr' },
			    { text: 'Quoted PM to '+$scope.users[message.Creator].UName, act: 'qpm' } ];
	    if (typeof message.Score != 'undefined') {
		if (!message.PlusOnly) buttons.push( { text: '<span class="red"><i class="icon ion-minus"></i> Neg</span>', act: 'neg' } );
		buttons.push( { text: '<span class="green"><i class="icon ion-plus"></i> Plus</span>', act: 'plus' } );
	    }
	    if (message.Creator == $rootScope.$storage.uuid) {
		buttons.push( { text: 'Edit this post', act: 'edit' } );
	    }
	    var opts = {
		titleText: 'Post [#'+message.count+'] Actions',
		buttons: buttons,
		cancelText: 'Cancel',
		buttonClicked: function(index) {
		    var id = 'ta'+$scope.ftuid;
		    var act = buttons[index].act;
		    if (act === 'mention') {
			document.getElementById(id).value = '@'+$scope.users[message.Creator].UName+' '+document.getElementById(id).value;
		    } else if (act === 'qr') {
			$scope.forumPost.text = '**'+$scope.users[message.Creator].UName+' wrote:**'+"\n"+message.Replyify+"\n";
		    } else if (act === 'qpm') {
			$scope.userPM($scope.users[message.Creator].UName, $scope.RawSubject, message.Replyify);
		    } else if (act === 'neg') {
			$scope.sendScore(message.ID, $rootScope.$storage.uuid, message.ScoreKey, -1);
		    } else if (act === 'plus') {
			$scope.sendScore(message.ID, $rootScope.$storage.uuid, message.ScoreKey, 1);
		    } else if (act === 'edit') {
			$scope.editPost(message.ID, message.RawMessage, message.FirstPost, message.HasDisable,
					message.NoScores, message.NoReplies, $scope.RawSubject);
		    }
		    return true;
		}
	    };

	    if (message.Creator == $rootScope.$storage.uuid) {

		opts.destructiveText = 'Delete this post';

		opts.destructiveButtonClicked = function() {

		    var popbuttons = [ { text: 'Dismiss', type: 'button-dark' } ];
		    popbuttons.push( { text: 'Yes, delete this post', type: 'button-assertive',
				       onTap: function(e) {

					   $ionicLoading.show( { duration: 3000 });
					   $rootScope.error = null;
					   $http.post($rootScope.$storage.baseURL,
						      { 'u'     : $rootScope.$storage.username,
							'hash'  : $rootScope.$storage.hash,
							'A'     : 'JSON',
							'av'    : AppVersion,
							'type'  : 'ForumPostDelete',
							'ftuid' : $scope.ftuid,
							'fmuid' : message.ID,
						      })
					       .success(function(data, status, headers, config) {
						   if (data.error) {
						       $rootScope.error = data.error;
						       if (data.badsignon) { $scope.badSignOn(); }
						   } else {
						       $scope.messages.splice($scope.messages.indexOf(message), 1);
						   }
						   $ionicLoading.hide();
					       })
					       .error(function(data, status, headers, config) {
						   $rootScope.error = "Failed to delete forum post!";
						   if (!$rootScope.online) $rootScope.error = $rootScope.error+' Network is offline.';
						   $ionicLoading.hide();
					       });
				       }
				     });
		    var notifyPopup = $ionicPopup.show({
			title: 'Delete Forum Post',
			template: 'Are you sure you want to delete this forum post?',
			scope: $scope,
			buttons: popbuttons
		    });
		    return true;
		}
	    };
    	    $ionicActionSheet.show(opts);
	};
	
	$scope.post = function() {
	    $ionicLoading.show( { duration: 3000 });
	    $rootScope.error = null;
	    $http.post($rootScope.$storage.baseURL,
		       { 'u'     : $rootScope.$storage.username,
			 'hash'  : $rootScope.$storage.hash,
			 'A'     : 'JSON',
			 'av'    : AppVersion,
			 'type'  : 'ForumPost',
			 'topic' : $scope.ftuid,
			 'msg'   : $scope.forumPost.text,
		       })
		.success(function(data, status, headers, config) {
		    if (data.error) {
			$rootScope.error = data.error;
			if (data.badsignon) { $scope.badSignOn(); }
		    } else {
			$scope.forumPost = { text: null };
			$scope.messages = [ ];
			$scope.next = 0;
			$scope.loadMore(0,0);
                    }
		    $ionicLoading.hide();
		})
		.error(function(data, status, headers, config) {
		    $rootScope.error = "Failed to Post Forum Message!";
		    if (!$rootScope.online) $rootScope.error = $rootScope.error+' Network is offline.';
		    $ionicLoading.hide();
		});
	};

	$scope.keepNew = function() {
	    $ionicLoading.show( { duration: 3000 });
	    $rootScope.error = null;
	    $http({
		url: $rootScope.$storage.baseURL,
		method: 'POST',
		data: {
		    'u'     : $rootScope.$storage.username,
		    'hash'  : $rootScope.$storage.hash,
		    'A'     : 'JSON',
		    'av'    : AppVersion,
		    'type'  : 'TopicKeepNew',
		    'ftuid' : $scope.ftuid,
		}
	    }).then(function(data) {
		if (data.data.error) {
		    $rootScope.error = data.data.error;
		    if (data.data.badsignon) { $scope.badSignOn(); }
		} else {
		    $scope.updateCounts(data.data);
		    $rootScope.$broadcast('topic.keepNew', $scope.area, $scope.ftuid);
		    var goparams = {};
		    goparams['area'] = $scope.area;
                    $state.go('app.forum', goparams);
		}
		$ionicLoading.hide();
	    }, function(data) { 
		$rootScope.error = "Failed to Keep New Topic!";
		$ionicLoading.hide();
	    });
	};

	$scope.loadMore = function(infin, refresh) {
	    if (refresh) {
		$scope.messages = [ ];
		$scope.next = 0;
		$scope.moreItems = true;
	    } else if (!infin) $ionicLoading.show( { duration: 3000 });
	    $rootScope.error = null;
	    var url = $rootScope.$storage.baseURL+$rootScope.jsonURLBase+'&type=topic&topic='+$scope.ftuid+$rootScope.loginURL()+'&next='+$scope.next;
	    $scope.logger('topic url = '+url);
	    $http.get(url).
		success(function(data, status, headers, config) {
		    if (data.error) {
			$rootScope.error = data.error;
			$scope.moreItems = false;
			$scope.NoPost = 'error';
			if (data.badsignon) { $scope.badSignOn(); }
		    } else {
			$scope.messages = $scope.messages.concat(data.messages);
			for (var uuid in data.users) {
			    $scope.users[uuid] = data.users[uuid];
			}
			for (var i in data.messages) {
			    if ($scope.users[$scope.messages[i].Creator] &&
				$scope.users[$scope.messages[i].Creator].Avatar) {
				$scope.messages[i].src = ImageHost+$scope.users[$scope.messages[i].Creator].Avatar;
			    } else {
				$scope.messages[i].src = false;
			    }
			}
			$scope.Subject = data.Subject;
			$scope.RawSubject = data.RawSubject;
			$scope.area = data.area;
			$scope.NoPost = data.NoPost;
			if (!$rootScope.login.signedOn) $scope.NoPost = 'Sign On to post.';

			$rootScope.$broadcast('topic.clearNew', $scope.ftuid);
			if (typeof data.messages == 'undefined') {
			    $scope.moreItems = false;
			} else {
			    if (data.messages.length < data.maxRecords) $scope.moreItems = false;
			    $scope.next += data.messages.length;
			}
		    }
		    if (refresh) $scope.$broadcast('scroll.refreshComplete');
		    else if (infin) $scope.$broadcast('scroll.infiniteScrollComplete');
		    else $ionicLoading.hide();
		}).
		error(function(data, status, headers, config) {
		    $scope.NoPost = 'error';
		    $rootScope.error = "Failed to get Forum Messages!";
		    if (!$rootScope.online) $rootScope.error = $rootScope.error+' Network is offline.';
		    if (refresh) $scope.$broadcast('scroll.refreshComplete');
		    else if (infin) $scope.$broadcast('scroll.infiniteScrollComplete');
		    else $ionicLoading.hide();
		});
	};
	$scope.loadMore(0,0);
    })

    .controller('user', function($scope, $http, $state, $ionicLoading, $rootScope) {
	$rootScope.error = null;
	$scope.info = { };
	
	$scope.loadMore = function() {
	    $ionicLoading.show( { duration: 3000 });
	    $rootScope.error = null;
	    var url = $rootScope.$storage.baseURL+$rootScope.jsonURLBase+'&type=user&un='+$rootScope.userPageUName+$rootScope.loginURL();
	    $scope.logger('user url = '+url);
	    $http.get(url).
		success(function(data, status, headers, config) {
		    if (data.error) {
			$rootScope.error = data.error;
			if (data.badsignon) { $scope.badSignOn(); }
		    } else {
			data.info.LastAgo = data.info.LastSQL.replace(' ', 'T');
			$scope.info = data.info;
		    }
		    $ionicLoading.hide();
		}).
		error(function(data, status, headers, config) {
		    $rootScope.error = "Failed to get user info!";
		    if (!$rootScope.online) $rootScope.error = $rootScope.error+' Network is offline.';
		    $ionicLoading.hide();
		});
	};
	$scope.loadMore();
    })

    .controller('editpost', function($scope, $http, $state, $ionicLoading, $rootScope, $ionicHistory) {
	$rootScope.error = null;

	$scope.post = function() {
	    $ionicLoading.show( { duration: 3000 });
	    $rootScope.error = null;
	    $http.post($rootScope.$storage.baseURL,
		       { 'u'     : $rootScope.$storage.username,
			 'hash'  : $rootScope.$storage.hash,
			 'A'     : 'JSON',
			 'av'    : AppVersion,
			 'type'  : 'EditForumPost',
			 'fmuid' : $scope.fmuid,
			 'subj'  : $scope.m.subject,
			 'msg'   : $scope.m.text,
			 'noscr' : $scope.m.noscores?1:null,
			 'norep' : $scope.m.noreplies?1:null,
		       })
		.success(function(data, status, headers, config) {
		    if (data.error) {
			$rootScope.error = data.error;
			if (data.badsignon) { $scope.badSignOn(); }
		    } else {
			$scope.editPostModal.remove();
			$rootScope.$broadcast('post.update', data);
                    }
		    $ionicLoading.hide();
		})
		.error(function(data, status, headers, config) {
		    $rootScope.error = "Failed to Edit Forum Message!";
		    if (!$rootScope.online) $rootScope.error = $rootScope.error+' Network is offline.';
		    $ionicLoading.hide();
		});
	};
    })

    .controller('newtopic', function($scope, $http, $state, $ionicLoading, $rootScope, $ionicHistory) {
	$rootScope.error = null;
	$scope.m = { subject: null, text: null };

	$scope.post = function() {
	    $ionicLoading.show( { duration: 3000 });
	    $rootScope.error = null;
	    $http.post($rootScope.$storage.baseURL,
		       { 'u'     : $rootScope.$storage.username,
			 'hash'  : $rootScope.$storage.hash,
			 'A'     : 'JSON',
			 'av'    : AppVersion,
			 'type'  : 'newtopic',
			 'area'  : $scope.area,
			 'subj'  : $scope.m.subject,
			 'msg'   : $scope.m.text,
			 'noscr' : $scope.m.noscores?1:null,
			 'norep' : $scope.m.noreplies?1:null,
		       })
		.success(function(data, status, headers, config) {
		    if (data.error) {
			$rootScope.error = data.error;
			if (data.badsignon) { $scope.badSignOn(); }
		    } else {
			$scope.newtopicModal.remove();
			$rootScope.$broadcast('post.new', data);
                    }
		    $ionicLoading.hide();
		})
		.error(function(data, status, headers, config) {
		    $rootScope.error = "Failed to Create Forum Topic!";
		    if (!$rootScope.online) $rootScope.error = $rootScope.error+' Network is offline.';
		    $ionicLoading.hide();
		});
	};
    })

    .controller('chat', function($scope, $http, $rootScope, chatsock, $timeout) {
	
	$rootScope.$on("chatsock:addMessage", function(event, line) {
	    var lr = line.split(" "); lr.shift();
	    var when = lr.shift();
	    var who = lr.shift();
	    var whisperto = lr.shift();
	    var whispertoname = lr.shift();
	    var msg = lr.join(" ");
	    var sepact = ": ";
	    var whisp = "";
	    var pre = "";
	    if (whisperto != 0 && whisperto != 2 && who == $rootScope.$storage.username) { whisp = " whispers to "+whispertoname; }
	    if (whisperto == $rootScope.$storage.uuid) whisp = " whispers to you";
	    if (whisperto == 2) { pre = "* "; sepact = " "; }
	    var message = { User: who,
			    Created: moment.unix(when),
			    Message: msg };
	    $timeout(function() {
		$scope.messages.unshift(message);
		$scope.messages = $scope.messages.slice(0, 100); // Save only 100 messages
	    }, 10);
	}); 

	$rootScope.$on("chatsock:updateWhoList", function(event, line) {
	    var lr = line.split(" "); lr.shift();
	    var wholist = lr.shift();
	    $scope.whos = wholist.split(',')
	}); 
	
	$scope.sendChatMsg = function() {
	    chatsock.send($scope.chatMsg.text);
	    $scope.chatMsg.text = null;
	};

	$scope.reset = function(refresh) {
	    $scope.messages = [ ];
	    $scope.whos = [ ];
	    $rootScope.error = null;
	    $scope.chatMsg = { text: null };
	    chatsock.connect();
	    if (refresh) $timeout(function() { $scope.$broadcast('scroll.refreshComplete'); }, 10);
	};

	$scope.reset(0);
    })

    .controller('createaccount', function($scope, $http, $ionicLoading, $rootScope, $timeout) {

	$scope.$on("modal.shown", function(event){
	    $scope.formFiller();
	});
	
    	$scope.doCreate = function() {
	    $ionicLoading.show( { duration: 3000 });
	    $rootScope.error = null;
	    var plat = null;
	    if (typeof device !== 'undefined') plat = device.platform;
	    $http({
		url: $rootScope.$storage.baseURL,
		method: 'POST',
		data: {
		    'A'    : 'JSON',
		    'av'   : AppVersion,
		    'type' : 'CreateAccount',
		    'plat' : plat,		    
		    'un'   : $scope.m.un,
		    'fn'   : $scope.m.fn,
		    'ln'   : $scope.m.ln,
		    'em'   : $scope.m.em,
		    'co'   : $scope.m.selectedCountry.id,
		    'st'   : $scope.m.selectedState.id,
		    'tn'   : $scope.m.tn,
		}
	    }).then(function(data) {
		if (data.data.error) {
		    $rootScope.error = data.data.error;
		    if (data.data.badsignon) { $scope.badSignOn(); }
		} else {
		    $scope.m.successMsg = data.data.SuccessMsg;
		    $scope.m.oath = data.data.Oath;
		    $scope.logger('scope.m.successMsg = '+$scope.m.successMsg);
		    $scope.m.creating = false;
		    $scope.m.done = true;
		}
		$ionicLoading.hide();
	    }, function(data) { 
		$rootScope.error = "Failed to create account!";
		$ionicLoading.hide();
	    });
	};

	$scope.checkUN = function(un) {
	    $rootScope.error = null;
	    un.$setValidity("taken", true);
	    var url = $rootScope.$storage.baseURL+$rootScope.jsonURLBase+'&type=CheckUN&un='+encodeURIComponent($scope.m.un);
	    $scope.logger('CheckUN url = '+url);
	    $http.get(url).
		success(function(data, status, headers, config) {
		    // note: ignores errors
		    if (!data.error) {
			if (data.taken) un.$setValidity("taken", false);
		    }
		});
	};

	$scope.resetCreate = function() {
	    //form.un.$setValidity("taken", true);
	    $scope.m = {
		un:    null,
		fn:    null,
		ln:    null,
		em:    null,
		st:    null,
		tn:    null,
		limun: null,
		limfn: null,
		limln: null,
		limem: null,
		limtn: null,
		patun: /^[a-zA-Z0-9_]*$/,
		patfn: /^[a-zA-Z\s\.]*$/,
		patln: /^[a-zA-Z\s\.]*$/,
		pattn: /^[a-zA-Z\s\.]*$/,
		selectedCountry: { id: null, label: null },
		selectedState: { id: null, label: null },
		creating:   true,
		done:       false,
		oath:       null,
		successMsg: null,
	    };
	    if (typeof $scope.crform !== 'undefined') $scope.checkUN($scope.crform.un);
	};
	    
	$scope.formFiller = function() {
	    console.trace();
	    $rootScope.error = null;
	    var url = $rootScope.$storage.baseURL+$rootScope.jsonURLBase+'&type=CreateFormFills';
	    if (typeof $scope.m.selectedCountry.id !== 'undefined' && $scope.m.selectedCountry.id !== null) url = url + '&defcountry='+encodeURIComponent($scope.m.selectedCountry.id);
	    $scope.logger('CreateFormFills url = '+url);
	    $http.get(url).
		success(function(data, status, headers, config) {
		    if (data.error) {
			$rootScope.error = data.error;
			if (data.badsignon) { $scope.badSignOn(); }
		    } else {
			//$scope.logger('CreateFormFills = '+JSON.stringify(data));
			$scope.countries = data.Countries;
			$scope.m.selectedCountry.id = data.defcountry;
			$scope.m.selectedState.id = data.defstate;
			$scope.states = data.States;
			$scope.m.limun = data.LIM_UN;
			$scope.m.limfn = data.LIM_FN;
			$scope.m.limln = data.LIM_LN;
			$scope.m.limem = data.LIM_EA;
			$scope.m.limtn = data.LIM_TN;
		    }
		}).
		error(function(data, status, headers, config) {
		    $rootScope.error = "Failed to get create account form info!";
		});
	};
	$scope.resetCreate();
    })
	
    .controller('userpm', function($scope, $http, $state, $ionicLoading, $rootScope, $ionicHistory) {
	$rootScope.error = null;
	$scope.subject;
	$scope.toUName = $rootScope.toUName;
	$scope.pmMsg;
	
	$scope.sendPM = function() {
	    $ionicLoading.show( { duration: 3000 });
	    $rootScope.error = null;
	    $http({
		url: $rootScope.$storage.baseURL,
		method: 'POST',
		data: {
		    'u'       : $rootScope.$storage.username,
		    'hash'    : $rootScope.$storage.hash,
		    'A'       : 'JSON',
		    'av'      : AppVersion,
		    'type'    : 'SendPM',
		    'conv'    : 'new',
		    'subject' : $scope.subject.text,
		    'to'      : $scope.toUName,
		    'msg'     : $scope.pmMsg.text,
		}
	    }).then(function(data) {
		if (data.data.error) {
		    $rootScope.error = data.data.error;
		    if (data.data.badsignon) { $scope.badSignOn(); }
		} else {
		    var goplace = null;
		    var goparams = {};
		    $scope.userPMModal.remove();
		    goplace = 'app.pm';
		    goparams['convid'] = data.data.pmid;
		    $ionicHistory.clearCache().then(function(){ $state.go(goplace, goparams) });
                }
		$ionicLoading.hide();
	    }, function(data) { 
		$rootScope.error = "Failed to send new Private Message!";
		$ionicLoading.hide();
	    });
	};
    })

    .controller('pm', function($scope, $http, $stateParams, $location, $ionicScrollDelegate, $state, $ionicLoading, $rootScope) {
	$scope.messages = [ ];
	$rootScope.error = null;
	$scope.pmid = $stateParams.convid;
	$scope.pmMsg = { text: null };
	$scope.fromuser = null;
	$scope.subject = null;
	$scope.moreItems = true;
	$scope.next = 0;

	$scope.sendPM = function() {
	    $ionicLoading.show( { duration: 3000 });
	    $rootScope.error = null;
	    $http({
		url: $rootScope.$storage.baseURL,
		method: 'POST',
		data: {
		    'u'    : $rootScope.$storage.username,
		    'hash' : $rootScope.$storage.hash,
		    'A'    : 'JSON',
		    'av'   : AppVersion,
		    'type' : 'SendPM',
		    'conv' : $scope.pmid,
		    'msg'  : $scope.pmMsg.text,
		}
	    }).then(function(data) {
		if (data.data.error) {
		    $rootScope.error = data.data.error;
		    if (data.data.badsignon) { $scope.badSignOn(); }
		} else {
		    $scope.pmMsg = { text: null };
		    $scope.messages = [ ];
		    $scope.next = 0;
		    $scope.loadMore(0,0);
                }
		$ionicLoading.hide();
	    }, function(data) { 
		$rootScope.error = "Failed to send Private Message!";
		$ionicLoading.hide();
	    });
	};

	$scope.keepNew = function() {
	    $ionicLoading.show( { duration: 3000 });
	    $rootScope.error = null;
	    $http({
		url: $rootScope.$storage.baseURL,
		method: 'POST',
		data: {
		    'u'    : $rootScope.$storage.username,
		    'hash' : $rootScope.$storage.hash,
		    'A'    : 'JSON',
		    'av'   : AppVersion,
		    'type' : 'PMKeepNew',
		    'conv' : $scope.pmid,
		}
	    }).then(function(data) {
		if (data.data.error) {
		    $rootScope.error = data.data.error;
		    if (data.data.badsignon) { $scope.badSignOn(); }
		} else {
		    $scope.updateCounts(data.data);
		    $rootScope.$emit('pm.keepNew', $scope.pmid);
                    $state.go('app.pms');
		}
		$ionicLoading.hide();
	    }, function(data) { 
		$rootScope.error = "Failed to Keep New Private Message!";
		$ionicLoading.hide();
	    });
	};

	$scope.loadMore = function(infin, refresh) {
	    if (refresh) {
		$scope.messages = [ ];
		$scope.next = 0;
		$scope.moreItems = true;
	    } else if (!infin) $ionicLoading.show( { duration: 3000 });
	    $rootScope.error = null;
	    var url = $rootScope.$storage.baseURL+$rootScope.jsonURLBase+'&type=pm&conv='+$scope.pmid+$rootScope.loginURL()+'&next='+$scope.next;
	    $scope.logger('pm url = '+url);
	    $http.get(url).
		success(function(data, status, headers, config) {
		    if (data.error) {
			$rootScope.error = data.error;
			$scope.moreItems = false;
			if (data.badsignon) { $scope.badSignOn(); }
		    } else {
			$scope.fromuser = data.FromUser;
			$scope.touser = data.ToUser;
			$scope.subject  = data.Subject;
			$scope.updateCounts(data);
			$rootScope.$emit('pm.clearNew', $scope.pmid);
			if (data.FromUser == 'system') $scope.NoPost = true;
			if (typeof data.messages == 'undefined') {
			    $scope.moreItems = false;
			} else {
			    if (data.messages.length < data.maxRecords) $scope.moreItems = false;
			    $scope.next += data.messages.length;
			    $scope.messages = $scope.messages.concat(data.messages);		
			}
                    }
		    if (refresh) $scope.$broadcast('scroll.refreshComplete');
		    else if (infin) $scope.$broadcast('scroll.infiniteScrollComplete'); 
		    else $ionicLoading.hide();
		}).
		error(function(data, status, headers, config) {
		    $rootScope.error = "Failed to get Private Message!";
		    if (!$rootScope.online) $rootScope.error = $rootScope.error+' Network is offline.';
		    if (refresh) $scope.$broadcast('scroll.refreshComplete');
		    else if (infin) $scope.$broadcast('scroll.infiniteScrollComplete'); 
		    else $ionicLoading.hide();
		});
	};
	$scope.loadMore(0,0);
    });

